from __future__ import unicode_literals

import os, datetime

#from django.db import connection

from media_explorer.fields import MediaField, RichTextField

from django.db import models

class Example(models.Model):
    """
    Example model that uses MediaField and RichTextField
    """

    name = models.CharField(null=True,blank=True)
    lead_media = MediaField()
    image = MediaField(type="image")
    video = MediaField(type="video")
    body = RichTextField()

    class Meta:
        verbose_name_plural = "Example"

    def __unicode__(self):
        return u"%s" % (self.name)




from django import forms
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from ckeditor.widgets import CKEditorWidget

class MediaWidget(forms.Widget):
    template_name = 'admin/media_field.html'

    class Media:
        js = (
            'js/vendor/jquery/jquery-1.11.2.min.js',
            'js/media_explorer/media_explorer.js',
            'js/media_field_admin.js',
        )
        css = {
            'all': (
                'css/hide_media_fields.css',
            )
        }

    def render(self, name, value, attrs=None):
        context = {"name":name,"value":value}
        return mark_safe(render_to_string(self.template_name, context))


class RichTextWidget(CKEditorWidget):

    class Media:
        js = (
            'js/vendor/jquery/jquery-1.11.2.min.js',
            'js/vendor/jQuery-Impromptu-6.1.0/jquery-impromptu.min.js',
            'js/media_explorer/media_explorer.js',
            'ckeditor/ckeditor/plugins/media_explorer/callback.js',
        )
        css = {
            'all': (
                'js/vendor/jQuery-Impromptu-6.1.0/jquery-impromptu.min.css',
            )
        }




import json, traceback, re
from django import template
from django.template import Template, Context, RequestContext
from django.conf import settings

from media_explorer.models import Element, Gallery, GalleryElement, ResizedImage
from django.template import Context
from django.template.loader import get_template
import traceback

register = template.Library()

def get_media_from_json(json_string):
    data = {}
    try:
        data = json.loads(json_string)
    except:
        pass

    return data


register.assignment_tag()(get_media_from_json)



import json
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from media_explorer.models import Element, Gallery
from media_explorer.forms import MediaFormField, RichTextFormField

def parse_media(string_or_obj):
    """Takes a JSON string, converts it into a Media object."""
    data = {}
    kwargs = {}
    kwargs["id"] = None
    kwargs["type"] = None
    kwargs["caption"] = None
    kwargs["credit"] = None
    try:
        if type(string_or_obj) is dict:
            data = string_or_obj
        elif type(string_or_obj) is Element:
            data["id"] = string_or_obj.id
            data["type"] = string_or_obj.type
            data["caption"] = string_or_obj.description
            data["credit"] = string_or_obj.credit
        elif type(string_or_obj) is Gallery:
            data["id"] = string_or_obj.id
            data["type"] = "gallery"
        else:
            data = json.loads(string_or_obj)
    except Exception as e:
        raise ValidationError("Media parsing error: " + str(e))

    if data:
        kwargs.update(data)
    return Media(**kwargs)

def parse_richtext(text):
    """Takes a string, converts it into a RichText object."""
    return RichText(text)


class Media(object):
    """The corresponding Python object for the Django MediaField."""
    def __init__(self,id=None,type=None,caption=None,credit=None):
        self.id = id
        self.type = type
        self.caption = caption
        self.credit = credit

    def to_dict(self):
        _dict = {}
        _dict["id"] = self.id
        _dict["type"] = self.type
        _dict["caption"] = self.caption
        _dict["credit"] = self.credit
        return _dict

    def __repr__(self):
        #return "[MediaField object]: id=%s, type=%s" % (self.id, self.type)
        return json.dumps(self.to_dict())

class MediaField(models.TextField):
    """The Django MediaField."""

    description = _("A Media Explorer custom model field")

    def __init__(self, id=None, type=None, \
            credit=None, caption=None, *args, **kwargs):
        self.id = id
        self.type = type
        self.caption = caption
        self.credit = credit

        kwargs['null'] = True
        kwargs['blank'] = True

        super(MediaField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(MediaField, self).deconstruct()
        del kwargs["null"]
        del kwargs["blank"]
        return name, path, args, kwargs

    def db_type(self, connection):
        return "text"

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value
        return parse_media(value)

    def do_validation(self, media):
        if self.type and media.type and self.type != media.type:
            raise ValidationError("Invalid media type selected for this MediaField instance. It expected a '%s' but got a '%s' instead." % (self.type, media.type))

        #Override id/credit/caption
        if self.id : media.id = self.id
        if self.type: media.type = self.type
        if self.caption : media.caption = self.caption
        if self.credit: media.credit = self.credit

        #Validate that the image/video is in the system
        if media.type in ["image","video"] and \
                not Element.objects.filter(id=media.id,type=media.type).exists():
            raise ValidationError("Invalid %s selected. The %s was not found." % (media.type, media.type))

        #Validate that the gallery is in the system
        if media.type == "gallery" and \
                not Gallery.objects.filter(id=media.id).exists():
            raise ValidationError("Invalid %s selected. The %s was not found." % (media.type, media.type))

        return media

    def to_python(self, value):
        if isinstance(value, Media):
            return value

        if value is None:
            return value

        return self.do_validation(parse_media(value))
    
    def get_prep_value(self, value):
        value_dict = {}

        try:
            value_dict["id"] = value.id
            value_dict["type"] = value.type
            value_dict["caption"] = value.caption
            value_dict["credit"] = value.credit
        except:
            pass

        if type(value) is Element:
            value_dict["id"] = value.id
            value_dict["type"] = value.type
            value_dict["caption"] = None
            value_dict["credit"] = None
            if value.description:
                value_dict["caption"] = value.description
            if value.credit:
                value_dict["credit"] = value.credit

        if value_dict:
            self.do_validation(parse_media(value_dict))
            return str(json.dumps(value_dict))

        if value: return str(value)
        return value

    def formfield(self, **kwargs):
        defaults = {}
        defaults["form_class"] = MediaFormField
        defaults.update(kwargs)
        return super(MediaField, self).formfield(**defaults)


class RichText(object):
    """The corresponding Python object for the Django RichTextField."""

    def __init__(self,text):
        self.text = text

    def __repr__(self):
        return self.text

class RichTextField(models.TextField):
    """The Django RichTextField."""

    description = _("A RichText Explorer custom model field")

    def __init__(self, *args, **kwargs):
        kwargs['null'] = True
        kwargs['blank'] = True
        super(RichTextField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(RichTextField, self).deconstruct()
        del kwargs["null"]
        del kwargs["blank"]
        return name, path, args, kwargs

    def db_type(self, connection):
        return "text"

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value
        return parse_richtext(value)

    def do_validation(self, richtext):
        return richtext

    def to_python(self, value):
        if isinstance(value, RichText):
            return value

        if value is None:
            return value

        return self.do_validation(parse_richtext(value))
    

    def get_prep_value(self, value):
        if value: return str(value)
        return value

    def formfield(self, **kwargs):
        defaults = {}
        defaults["form_class"] = RichTextFormField
        defaults.update(kwargs)
        return super(RichTextField, self).formfield(**defaults)



import json
from django.core.exceptions import ValidationError
from django import forms
from media_explorer.widgets import MediaWidget, RichTextWidget

class MediaFormField(forms.Field):
    def __init__(self, name=None, required=False, widget=MediaWidget, label=None, initial=None, help_text="", max_length=None, *args, **kwargs):
        super(MediaFormField, self).__init__(required=required, widget=MediaWidget, label=label, initial=initial, help_text=help_text, *args, **kwargs)

    def clean(self, value):
        if value:
            try:
                data = json.loads(value)
            except Exception as e:
                raise ValidationError("JSON parsing error: " + str(e))
            return value
        return None

class RichTextFormField(forms.Field):
    def __init__(self, name=None, required=False, widget=RichTextWidget, label=None, initial=None, help_text="", max_length=None, *args, **kwargs):
        super(RichTextFormField, self).__init__(required=required, widget=RichTextWidget, label=label, initial=initial, help_text=help_text, *args, **kwargs)

    def clean(self, value):
        if value:
            return value
        return None



from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.views.generic import TemplateView

from media_explorer.views import ElementStatsView, GalleryStatsView
from rest_framework import routers
from media_explorer.models import Element
from media_explorer.django_rest_framework import ElementList, ElementDetail, GalleryList, GalleryDetail, GalleryElementDetail, ResizedImageList

urlpatterns = patterns('',
    url(r'^api/stats/elements', ElementStatsView.as_view(), name='api-stats-elements'),
    url(r'^api/stats/galleries', GalleryStatsView.as_view(), name='api-stats-galleries'),
    url(r'^api/media/elements/(?P<pk>[0-9]+)$', ElementDetail.as_view()),
    url(r'^api/media/elements', ElementList.as_view(), name='api-media-elements'),
    url(r'^api/media/resizedimages', ResizedImageList.as_view(), name='api-media-resizedimages'),
    url(r'^api/media/galleries/(?P<pk>[0-9]+)$', GalleryDetail.as_view()),
    url(r'^api/media/galleries', GalleryList.as_view(), name='api-media-galleries'),
    url(r'^api/media/galleryelements/(?P<pk>[0-9]+)$', GalleryElementDetail.as_view()),
    url(r'^media_explorer/', staff_member_required(TemplateView.as_view(template_name='admin/media_explorer/base.html')), name='media_explorer')
)



import json, traceback, re
from django import template
from django.template import Template, Context, RequestContext
from django.db.models import Q
from django.conf import settings

from media_explorer.models import Element, Gallery, GalleryElement, ResizedImage
from django.template import Context
from django.template.loader import get_template
import traceback

register = template.Library()

def get_inline_image(id):
    try:
        t = get_template("media_explorer/inline_image.html")
        image = Element.objects.get(id=id,type="image")
        c = Context({"image":image,"class":"img-inline-float-left"})
        return t.render(c)
    except:
        print traceback.format_exc()

    return ""


def get_video(id):
    try:
        t = get_template("media_explorer/video.html")
        video = Element.objects.get(id=id,type="video")
        c = Context({"video":video})
        return t.render(c)
        #return video.video_embed
    except:
        print traceback.format_exc()

    return ""

def get_media_gallery(id,template="media_explorer/gallery_slick.html"):
    try:
        t = get_template(template)
        gallery = Gallery.objects.get(id=id)
        galleryelements = GalleryElement.objects.filter(gallery=gallery)
        c = Context({"gallery":gallery,"galleryelements":galleryelements})
        return t.render(c)
    except:
        print traceback.format_exc()

    return ""

def has_size(element, comma_separated_sizes):
    try:
        args = comma_separated_sizes.split(",")
        if ResizedImage.objects.filter(image=element,size__in=args).exists():
            return True
        for size in args:
            if ResizedImage.objects.filter(image=element,size=size).exists():
                return True
    except:
        print traceback.format_exc()

    return False

def get_image_url_from_size(id, *args):
    """
    The command may be {% get_image_url_from_size 123 "image|video|gallery" "800x500" "420x230" %}
    Or it could be  {% get_image_url_from_size 123 "800x500" "420x230" %}
    """
    try:
        if args[0] == "gallery":
            gallery = Gallery.objects.get(id=id)
            return gallery.thumbnail_image_url
        elif args[0] == "video":
            video = Element.objects.get(id=id)
            return video.thumbnail_image_url
    except:
        print traceback.format_exc()

    try:
        #If we are here then this is an image
        element = Element.objects.get(id=id)
        try:
            if not ResizedImage.objects.filter(image=element,size__in=args).exists():
                return element.image_url
            for size in args:
                if ResizedImage.objects.filter(image=element,size=size).exists():
                    return ResizedImage.objects.get(image=element,size=size).image_url
        except:
            print traceback.format_exc()
        return element.image_url
    except:
        print traceback.format_exc()


def show_short_code(html):
    try:

        pattern_img = r'\[media-explorer-image-(?P<id>\d+)\]?'
        pattern_video = r'\[media-explorer-video-(?P<id>\d+)\]?'
        pattern_gallery = r'\[media-explorer-gallery-(?P<id>\d+)\]?'

        match_img = re.findall(pattern_img, html, re.DOTALL)
        match_video = re.findall(pattern_video, html, re.DOTALL)
        match_gallery = re.findall(pattern_gallery, html, re.DOTALL)

        if match_img:
            for id in match_img:
                html2 = get_inline_image(id)
                html = re.sub(pattern_img,html2,html)

        if match_video:
            for id in match_video:
                html2 = get_video(id)
                html = re.sub(pattern_video,html2,html)

        if match_gallery:
            for id in match_gallery:
                html2 = get_media_gallery(id)
                html = re.sub(pattern_gallery,html2,html)

        return html
    except:
        print traceback.format_exc()

    return ""

register.filter('show_short_code', show_short_code)
register.filter('has_size', has_size)
register.simple_tag()(get_media_gallery)
register.simple_tag()(get_video)
register.simple_tag()(get_inline_image)
register.simple_tag()(get_image_url_from_size)
